package PiTube;

use Mojo::Base 'Mojolicious';
use Mojo::Pg;
use Net::OAuth2::Profile::WebServer;
use Mojolicious::Plugin::Authentication;
use Mojolicious::Plugin::Authorization;
use PiTube::Schema;

sub startup {
    my $self = shift;

    # Nacteni konfigurace
    my $cfg = $self->plugin('Config');
    $self->helper( cfg => sub { return $cfg; } );

    # Podpis pro cookies
    $self->secrets($cfg->{session}{secrets});

    # Delka session
    $self->sessions->default_expiration($cfg->{session}{lifetime});

    $self->plugin('PiTube::Helpers::OAuth2');

    # migrace schematu
    my $pg = Mojo::Pg->new
        ->dsn($cfg->{database}{dsn})
        ->username($cfg->{database}{user})
        ->password($cfg->{database}{password})
    ;
    $pg->migrations->from_file($self->home . '/sql/migrations.sql');
    $pg->migrations->migrate();

    # Spojeni s databazi
    my $schema = PiTube::Schema->connect($cfg->{database});
    $self->helper( schema => sub { return $schema; } );

    $self->plugin('authentication', {
        autoload_user => 1,
        load_user => sub {
            my $c = shift;
            return $c->session->{user};
        },
        validate_user => sub {
            my $c = shift;
            return undef if ! $c->session->{user};
            return $c->session->{user}{id};
        },
    });

    $self->plugin('Authorization' => {
        is_role     => sub {
            my ($c, $role, $extradata) = @_;
            return 0 if ! $c->session->{user};
            my %user_roles = map { $_ => 1 } @{ $c->session->{user}{roles} };
            return exists $user_roles{$role} ? 1 : 0;
        },
        user_privs  => sub {},
        has_priv    => sub {},
        user_role   => sub {},
        fail_render => { status => 401, json => {} },
    });

    # defautni globalni promenne ve stash
    $self->defaults();

    # router
    my $r = $self->routes;

    $r->get('/')->to(cb => sub { shift->render('live'); });
    $r->get('/streams')->to(cb => sub { shift->render('streams'); });

    $r->get('/api/streams')->to('Stream#list');

    $r->get('/streams/:id')->to('Stream#info');
#    $r->get('/streams/add')->over( is => 'administrator' )->to('Stream#add');
#    $r->get('/streams/:id/form')->over( is => 'administrator'  )->to('Stream#form');
#    $r->post('/streams')->over( is => 'administrator'  )->to('Stream#create');
#    $r->put('/streams/:id')->over( is => 'administrator'  )->to('Stream#update');
#    $r->delete('/streams/:id')->over( is => 'administrator'  )->to('Stream#delete');

    $r->get('/oauth2')->to('OAuth2#callback');
    $r->get('/logout')->to('OAuth2#do_logout');
    $r->post('/callback/rtmp')->to('Nginx#callback_rtmp');
    $r->get('/callback/hls')->to('Nginx#callback_hls');
    $r->get('/play/:key')->to('Stream#player');

}

1;
